/*	The purpose of this exercise is as a motivation to conceive,
 * that a synchronous, single-threaded function can exist, which
 * would be called by a framework, once per output sample, and which
 * would return the per-sample value of the excitation codeword.
 *
 * 	Since this function is being fed a frequency which it will
 * keep accurate relative to an assumed 8kHz sample-rate, and units
 * of half-cycles per 160-sample frame, being fed an allocated
 * bit-field to store its excitation pulses in, but being fed little
 * else, it can be said running non-deterministically, yet always
 * consuming limited time in CPU cycles.
 *
 * 	The value returned by this function is to be multiplied by an
 * amplitude, by the calling framework, and added to a Linear
 * Predictive feedback loop, after which the sum is to be fed to
 * a low-pass filter, again, by the calling framework.
 *
 * 	This makes the function defined in this code a sort of oscillator.
 *
 * 	This source file differs from some of the others, in that it
 * cannot be compiled into an executable all by itself. After being
 * compiled, it will need to be linked to 'excite_selector_1.o' .
 *
 */

#include <stdint.h>

#include "CELP_osc.h"
#include "excite_selector.h"


static int osc_accum = 0;
static int osc_active = 0;
static int osc_ptr = 0;
static int osc_amp = 0;


int osc_play_codew(uint32_t bit_field[],
                   int length)
{
    int flag = 0;

    if (! osc_active) return 0;

    flag = read_excite_buffer(osc_ptr, bit_field);
    osc_ptr += 1;
    if (osc_ptr >= length) {
        osc_active = 0;
        osc_ptr = 0;
    }

    return flag * osc_amp;
}

int osc_trigger(uint32_t bit_field[],
                int length,
                int noise,
                int amp)
{
    uint16_t selector = 0x0100;
    pack_selector(length, &selector);
    expand_selector(selector, bit_field);
    noise_excite_buffer(noise, bit_field);
    osc_ptr = 0;
    osc_active = 1;
    osc_amp = amp;

    return 0;
}

int osc_adv_freq(int frequency,
                 uint32_t bit_field[],
                 int length,
                 int noise,
                 int amp)
{
    osc_accum += frequency;
    if (osc_accum >= 160) {
        osc_trigger(bit_field, length, noise, amp);
        osc_accum -= 320;
    }

    if (frequency == 1 && (! osc_active)) {
        osc_trigger(bit_field, 160, noise, amp);
        osc_accum = -160;
    }

    return 0;
}

int osc_gen(int frequency,
            uint32_t bit_field[],
            int length,
            int noise,
            int amp)
{
    osc_adv_freq(frequency, bit_field, length, noise, amp);

    return osc_play_codew(bit_field, 160);
}
